﻿# OCS P2P Conversation Archiving Reporter, v1.31, 2/26/10
#
# Author: Sasa Juratovic (sasaj@microsoft.com)
# Author: Scott Bueffel (scott@flobee.net)
# Scott's updates group by conversation, remove toast notifications, and convert to html internally
# Updated to support OCS 2007 R2 by: Nick Smith (karsmith@microsoft.com)
# Tested with PowerShell v1.0 RTW and OCS 2007 (b6362)
# Tested with PowerShell v1.0 RTW and OCS 2007 R2 (b6907)
# 
# Script will connect to user specified SQL Server and retreive IM messages. Result will be writen to console and IM.xml in local directory
#   1. If only one user is specified, all IM messages from/to that user will be retreived
#   2. If two users are specified, all IM messages between those two users will be retreived
#   3. Date parameter names do not need to be used only if two users specified.  Otherwise, use -start and/or -end
#	   Date filters are optional.  You can specify none, one, or both.
#
# Sample usage #1: .\get-ArchivingData.ps1 sqlserver\instance user1@contoso.com -start "2009-04-01" -end "2009-04-30"
# Sample usage #2: .\get-ArchivingData.ps1 sqlserver\instance user1@contoso.com user2@nwtraders.com -end "2009-08-31"
# Sample usage #3  .\get-ArchivingData.ps1 sqlserver\instance user1@contoso.com user2@nwtraders.com "2009-04-15" "2009-09-01"


param (
    [string]$sqlserver = $(throw "Please specify SQL Server name and instance e.g. sqlserver\instance"),
    [string]$User1 = $(throw "You have to specify at least one user"),
    [string]$User2,
	[string]$start,
	[string]$end
)

If ($start -ne '')
	{
	#Start date has been specified
	$startdate = $start
	$start = "and Messages.MessageIdTime >= '$start' "
	}
Else 
	{
	$startdate = "Oldest record in the database"
	}

If ($end -ne '')
	{
	#End date has been specified
	$enddate = $end
	$end = "and Messages.MessageIdTime <= '$end' "
	}
Else
	{
	$enddate = "Newest record in the database"
	}
	

If ($User2 -ne '') {
    # User2 has been specified
    write-host "Searching for P2P IMs between" $User1 "and" $User2
    $Title = "IM P2P Conversations between " + $User1 + " and " + $User2
	$Subtitle = "Timeframe: From " + $startdate + " To " + $enddate
	$query = "SELECT RANK() over(partition by SessionIdTime order by SessionIdTime, MessageIdTime) as 'Rank', `
		SessionIdTime, MessageIdTime, Body, ContentTypeId, [from], [to] `
		From `
		(SELECT SessionIdTime, MessageIdTime, Body, ContentTypeId, u1.UserUri as [from], u2.UserUri as [to] from Messages, `
		Users u1, Users u2 where Messages.FromId = u1.UserId and Messages.ToId = u2.UserId and u1.UserUri = '$User1' `
		and u2.UserUri = '$User2' and LcsLog.dbo.Messages.Toast Is Null " + $start + $end + `
		"UNION ALL SELECT ` 
		SessionIdTime, MessageIdTime, Body, ContentTypeId, u2.UserUri as [from], u1.UserUri as [to] from Messages, `
		Users u2, Users u1 where Messages.FromId = u2.UserId and Messages.ToId = u1.UserId and u1.UserUri = '$User1' `
		and u2.UserUri = '$User2' and LcsLog.dbo.Messages.Toast Is Null " + $start + $end + `
		") As dConversation"
} else {
    # User2 has not been specified
    write-host "Searching for P2P IMs from/to" $User1
    $Title = "IM P2P Conversations between " + $User1 + " and any other user"
	$Subtitle = "Timeframe: From " + $startdate + " To " + $enddate
	$query = "SELECT RANK() over(partition by SessionIdTime order by SessionIdTime, MessageIdTime) as 'Rank', `
		SessionIdTime, MessageIdTime, Body, ContentTypeId, [from], [to] `
		From `
		(SELECT SessionIdTime, MessageIdTime, Body, ContentTypeId, u1.UserUri as [from], u2.UserUri as [to] from Messages, `
		Users u1, Users u2 where Messages.FromId = u1.UserId and Messages.ToId = u2.UserId and u1.UserUri = '$User1' `
		and LcsLog.dbo.Messages.Toast Is Null " + $start + $end + `
		"UNION ALL SELECT ` 
		SessionIdTime, MessageIdTime, Body, ContentTypeId, u2.UserUri as [from], u1.UserUri as [to] from Messages, `
		Users u2, Users u1 where Messages.FromId = u2.UserId and Messages.ToId = u1.UserId and u1.UserUri = '$User1' `
		and LcsLog.dbo.Messages.Toast Is Null "  + $start + $end + `
		") As dConversation"
}

# Connect to SQL Server and execute query. Get all ContentTypes
$connstring = "server=$sqlserver;Integrated Security=SSPI;database=lcslog"
$dataset = New-Object "System.Data.Dataset" "MyDataSet"
$ContentTypeQuery = "SELECT * from ContentTypes"
$da = New-Object "System.Data.SqlClient.SqlDataAdapter" ($ContentTypeQuery, $connstring)
$da.Fill($dataset) | out-null
$dataset.Tables[0].TableName = "ContentTypes"
$ContentTypes = $dataset.Tables["ContentTypes"]

foreach ($ContentType in $ContentTypes) {
	if ($ContentType.ContentType -eq "text/plain") {$TextPlain = $ContentType.ContentTypeId}
	if ($ContentType.ContentType -eq "text/x-msmsgsinvite") {$TextXmsmsgsinvite = $ContentType.ContentTypeId}
	if ($ContentType.ContentType -eq "text/rtf") {$TextRtf = $ContentType.ContentTypeId}
	if ($ContentType.ContentType -eq "text/html") {$TextHtml = $ContentType.ContentTypeId}
}

$ContentTypes = $null
$dataset = $null
$da = $null

# Connect to SQL Server and execute query. Get all Messages
$connstring = "server=$sqlserver;Integrated Security=SSPI;database=lcslog"
$dataset = New-Object "System.Data.Dataset" "MyDataSet"
$da = New-Object "System.Data.SqlClient.SqlDataAdapter" ($query, $connstring)
$da.Fill($dataset) | out-null
$dataset.Tables[0].TableName = "Messages"
$messages = $dataset.Tables["Messages"]
# Use System.Windows.Forms.RichTextBox for RTF to Text conversion
[void][System.Reflection.Assembly]::Load("System.Windows.Forms, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089")
$rtf = new-object System.Windows.Forms.RichTextBox

# Create new XML Document Object
$contentxmlDom = new-object System.Xml.XmlDocument

# Delete existing IM.xml
remove-item -path IM.xml -ea SilentlyContinue

# IM.xml in local path is where we'll write IM conversation
pwd | % {[string]$LocalPath = $_.path}
$LocalPath = $LocalPath + "\IM.xml"
Add-Content -Path $LocalPath -Value '<?xml version="1.0" encoding="ISO-8859-2"?>' 
Add-Content -Path $LocalPath -Value "<IMConversation>"
Add-Content -Path $LocalPath -Value "<User1>$User1</User1>"
Add-Content -Path $LocalPath -Value "<User2>$User2</User2>"
Add-Content -Path $LocalPath -Value "<Title>$Title</Title>"
Add-Content -Path $LocalPath -Value "<Subtitle>$Subtitle</Subtitle>"

# For each SQL record get From, To and Body
$i = 0
foreach ($message in $messages) {
	$i++
	if ($message.ContentTypeID -eq $TextRtf) {
	    # Message Body is in RTF format
		$rtf.rtf = $message.Body
		$Body = $rtf.text
		}
	else {
		# Message Body is in Text, HTML, or invite
		$Body = $message.body
		}
	#Remove link tag enclosure so hyperlinks show up
	$Body = $Body -replace "<|>",""
		
	$DateTime = $message.MessageIdTime
	$From = $message.from
	$To = $message.to
	$Rank = $message.Rank
	$Session = $message.SessionIdTime
	Add-Content -Path $LocalPath -Value "<IM>"
	Add-Content -Path $LocalPath -Value "<Rank>$Rank</Rank>"
	Add-Content -Path $LocalPath -Value "<SessionTime>$Session</SessionTime>"
	Add-Content -Path $LocalPath -Value "<DateTime>$DateTime</DateTime>"
	Add-Content -Path $LocalPath -Value "<From>$From</From>"
	Add-Content -Path $LocalPath -Value "<To>$To</To>"
	Add-Content -Path $LocalPath -Value "<Body><![CDATA[$Body]]></Body>"
	Add-Content -Path $LocalPath -Value "</IM>"
	$iProgress = (($i / $messages.Rows.Count) * 100)
	Write-Progress -Activity "Phase 1: Converting message bodies to plain text" -PercentComplete $iProgress -Status "Progress:" -CurrentOperation "$i of $($messages.Rows.Count) messages completed."
	}

# Complete IM.xml
Add-Content -Path $LocalPath -Value "</IMConversation>"
write-host "IM.xml created in local directory"

#Convert XML to HTML
$sourceXML = [xml](Get-Content $LocalPath)
Remove-Item -path IM.html -ea SilentlyContinue
pwd | % {[string]$LocalPath = $_.path}
$LocalPath = $LocalPath + "\IM.html"
Add-Content -path $LocalPath -Value '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN"'
Add-Content -Path $LocalPath -Value '  "http://www.w3.org/TR/html4/loose.dtd">'
Add-Content -Path $LocalPath -Value "<html>"
Add-Content -Path $LocalPath -Value "<head>"
Add-Content -Path $LocalPath -Value "<title>IM Conversation Archive</title>"
Add-Content -Path $LocalPath -Value "</head>"
Add-Content -Path $LocalPath -Value "<body>"

Add-Content -Path $LocalPath -Value '<font size="2" face="Verdana">'
Add-Content -Path $LocalPath -Value "<h2 align=`"center`">$($sourceXML.IMConversation.Title)</h2>"
Add-Content -Path $LocalPath -Value "<h3 align=`"center`">$($sourceXML.IMConversation.Subtitle)</h2>"
Add-Content -Path $LocalPath -Value "</font>"
Add-Content -Path $LocalPath -Value '<table border="0" cellpadding="1" width="100%" style="FONT-SIZE:8pt;FONT-FAMILY:verdana">'
$j = 0
foreach($IM in $sourceXML.IMConversation.IM)
	{
	$j++
	if ($IM.Rank -eq "1")
		{
		Add-Content -Path $LocalPath -Value "</tbody></table><br>"
		Add-Content -Path $LocalPath -Value '<table border="1" cellpadding="1" width="100%" style="FONT-SIZE:8pt;FONT-FAMILY:verdana">'
		Add-Content -Path $LocalPath -Value '<thead><tr bgcolor="#C0C0C0" align="center">'
		Add-Content -Path $LocalPath -Value	'<th><font color="#0000FF">Session Time (UTC):</font></th>'
		Add-Content -Path $LocalPath -Value	'<th><font color="#0000FF">Message Time (UTC):</font></th>'
		Add-Content -Path $LocalPath -Value	'<th><font color="#0000FF">From:</font></th>'
		Add-Content -Path $LocalPath -Value	'<th><font color="#0000FF">To:</font></th>'
		Add-Content -Path $LocalPath -Value '<th width="40%"><font color="#0000FF">Message:</font></th>'
		Add-Content -Path $LocalPath -Value	"</tr></thead>"
		Add-Content -Path $LocalPath -Value	"<tbody><tr>"
		Add-Content -Path $LocalPath -Value	"<td rowspan=`"100`" valign=`"top`">$($IM.SessionTime)</td>"
		Add-Content -Path $LocalPath -Value	"<td>$($IM.DateTime)</td>"
		Add-Content -Path $LocalPath -Value	"<td>$($IM.From)</td>"
		Add-Content -Path $LocalPath -Value	"<td>$($IM.To)</td>"
		Add-Content -Path $LocalPath -Value	"<td>$($IM.Body.get_FirstChild().get_Data())</td>"
		Add-Content -Path $LocalPath -Value	"</tr>"
		}
	else
		{
		Add-Content -Path $LocalPath -Value	"<tr>"
		Add-Content -Path $LocalPath -Value	"<td>$($IM.DateTime)</td>"
		Add-Content -Path $LocalPath -Value	"<td>$($IM.From)</td>"
		Add-Content -Path $LocalPath -Value	"<td>$($IM.To)</td>"
		Add-Content -Path $LocalPath -Value	"<td>$($IM.Body.get_FirstChild().get_Data())</td>"
		Add-Content -Path $LocalPath -Value	"</tr>"
		}
	$jProgress = ($j / $sourceXML.IMConversation.IM.Count) * 100
	Write-Progress -Activity "Phase 2: Converting XML to HTML" -PercentComplete $jProgress -Status "Progress:" -CurrentOperation "$j of $($sourceXML.IMConversation.IM.Count) messages completed."
	}
Write-Progress -Activity "Phase 2: Converting XML to HTML" -Completed -Status "All messages converted."
Add-Content -Path $LocalPath -Value "</table>"
Add-Content -Path $LocalPath -Value "</body>"
Add-Content -Path $LocalPath -Value "</html>"
write-host "IM.html created in local directory"